/****************************************************************************
  SH7262 VDC3 eXg5
****************************************************************************/

#include "cpu.h"
#include "cpu_init.h"
#include "cpu_uart.h"
#include "tinylib.h"

extern unsigned short int mag_if_logo[];

/* TFT-LCD GPG3224012QS4 */
#define LCD_HORIZONTAL_PERIOD         408	/*  */
#define LCD_HORIZONTAL_DISPLAY_PERIOD 320	/* \ */
#define LCD_HORIZONTAL_BACK_PORCH      38	/* obN|[` */
#define LCD_HORIZONTAL_PULSE_WIDTH      1	/*  */
#define LCD_VERTICAL_PERIOD           262	/*  */
#define LCD_VERTICAL_DISPLAY_PERIOD   240	/* \ */
#define LCD_VERTICAL_BACK_PORCH        18	/* obN|[` */
#define LCD_VERTICAL_PULSE_WIDTH        1	/*  */

/* OtBbN1 */
#define GRA1_HORIZONTAL_SIZE          320	/* \TCY */
#define GRA1_VERTICAL_SIZE            240	/* \TCY */
#define GRA1_HORIZONTAL_POS             0	/* \ʒu */
#define GRA1_VERTICAL_POS               0	/* \ʒu */
#define GRA1_VRAM_BASE         0x3C078000	/* eʓRAM y[W4(240KB) */

/* OtBbN2 */
#define GRA2_HORIZONTAL_SIZE          160	/* \TCY */
#define GRA2_VERTICAL_SIZE            120	/* \TCY */
#define GRA2_HORIZONTAL_POS            80	/* \ʒu */
#define GRA2_VERTICAL_POS              60	/* \ʒu */
#define GRA2_VRAM_BASE         0x3C050000	/* eʓRAM y[W3(160KB) */

int main(void)
{
	int i, j;
	unsigned short int *gra_vram;
	unsigned short int *img;
	unsigned short int pix;

	/* CPU  */
	CPU_BusInitialize();

	/* VDC3  */
	*PGCR7 = 0x5A01;	/* LCD_DATA0 */
	*PGCR4 = 0x1111;	/* LCD_CLK, LCD_DE, LCD_HSYNC, LCD_VSYNC */
	*PGCR3 = 0x1111;	/* LCD_DATA15-12 */
	*PGCR2 = 0x1111;	/* LCD_DATA11-8 */
	*PGCR1 = 0x1111;	/* LCD_DATA7-4 */
	*PGCR0 = 0x1110;	/* LCD_DATA3-1 */

	*STBCR7 = *STBCR7 & ~(1 << 4);	/* VDC3ւ̃NbN */

	/* \䕔  */
	*SGMODE           = 0x00000001;
	*SYNCNT           = 0x010F000C;
	*PANEL_CLKSEL     = *PANEL_CLKSEL & ~(1 << 12);
	*PANEL_CLKSEL     = 0x00002008;		/* 6MHz(48MHz / 8)  6.4MHz(typ) */
	*PANEL_CLKSEL     = *PANEL_CLKSEL | (1 << 12);
	*SYN_SIZE         = (LCD_VERTICAL_PERIOD << 16) | (LCD_HORIZONTAL_PERIOD);
	*PANEL_VSYNC_TIM  = (0 << 16) | (LCD_VERTICAL_PULSE_WIDTH);
	*PANEL_HSYNC_TIM  = (0 << 16) | (LCD_HORIZONTAL_PULSE_WIDTH);
	*DE_SIZE          = (LCD_VERTICAL_DISPLAY_PERIOD << 16) | (LCD_HORIZONTAL_DISPLAY_PERIOD);
	*DE_START         = (LCD_VERTICAL_BACK_PORCH << 16) | (LCD_HORIZONTAL_BACK_PORCH);
	*GRA_VSYNC_TIM    = 0;

	/* OtBbNX1  */
	*GRCMEN1      = 0x00000002;	/* JgCAʃC[s */
	*GRCBUSCNT1   = 0x00000000;	/* RGB565 */
	*GROPSADR1    = GRA1_VRAM_BASE;
	*GROPSWH1     = (GRA1_VERTICAL_SIZE << 16) | (GRA1_HORIZONTAL_SIZE);
	*GROPSOFST1   = GRA1_HORIZONTAL_SIZE * 2;
	*GROPDPHV1    = ((LCD_VERTICAL_BACK_PORCH - 1 + GRA1_VERTICAL_POS) << 16)
	                    | (LCD_HORIZONTAL_BACK_PORCH - 16 + GRA1_HORIZONTAL_POS);
	*GROPBASERGB1 = 0x0000FFFF;	/* OtBbNX摜OF(White) */
	*GRCMEN1      = *GRCMEN1 | (1 << 31);	/* WX^l] */

	/* OtBbNX2  */
	*GRCMEN2      = 0x00000003;	/* JgCAʃC[ */
	*GRCBUSCNT2   = 0x00000010;	/* ARGB4444 */
	*GROPSADR2    = GRA2_VRAM_BASE;
	*GROPSWH2     = (GRA2_VERTICAL_SIZE << 16) | (GRA2_HORIZONTAL_SIZE);
	*GROPSOFST2   = GRA2_HORIZONTAL_SIZE * 2;
	*GROPDPHV2    = ((LCD_VERTICAL_BACK_PORCH - 1 + GRA2_VERTICAL_POS) << 16)
	                    | (LCD_HORIZONTAL_BACK_PORCH - 16 + GRA2_HORIZONTAL_POS);
	*GROPBASERGB2 = 0x00000000;	/* OtBbNX摜OF(ʃC[Ŗ) */
	*GRCMEN2      = *GRCMEN2 | (1 << 31);	/* WX^l] */

	/* OtBbNX1 `(J[o[) */
	gra_vram = GRA1_VRAM_BASE;
	for(j = 0; j < GRA1_VERTICAL_SIZE / 2; j++)
	{
		for(i = 0; i < GRA1_HORIZONTAL_SIZE; i++)
		{
			if(i < GRA1_HORIZONTAL_SIZE / 8)
				*gra_vram++ = 0x0000;	/* Black */
			else if(i < GRA1_HORIZONTAL_SIZE / 8 * 2)
				*gra_vram++ = 0x001F;	/* Blue */
			else if(i < GRA1_HORIZONTAL_SIZE / 8 * 3)
				*gra_vram++ = 0x07E0;	/* Green */
			else if(i < GRA1_HORIZONTAL_SIZE / 8 * 4)
				*gra_vram++ = 0x07FF;	/* Cyan */
			else if(i < GRA1_HORIZONTAL_SIZE / 8 * 5)
				*gra_vram++ = 0xF800;	/* Red */
			else if(i < GRA1_HORIZONTAL_SIZE / 8 * 6)
				*gra_vram++ = 0xF81F;	/* Magenta */
			else if(i < GRA1_HORIZONTAL_SIZE / 8 * 7)
				*gra_vram++ = 0xFFE0;	/* Yellow */
			else
				*gra_vram++ = 0xFFFF;	/* White */
		}
	}
	for(j = 0; j < GRA1_VERTICAL_SIZE / 2; j++)
	{
		for(i = 0; i < GRA1_HORIZONTAL_SIZE; i++)
		{
			*gra_vram++ = (((i & 0xF8) >> 3) << 11)
			            | (((i & 0xFC) >> 2) << 5)
			            |  ((i & 0xF8) >> 3);
		}
	}

	/* OtBbNX2 `(InterfaceS) */
	gra_vram = GRA2_VRAM_BASE;
	img = mag_if_logo;
	for(j = 0; j < GRA2_VERTICAL_SIZE; j++)
	{
		for(i = 0; i < GRA2_HORIZONTAL_SIZE; i++)
		{
			pix = *img++;
			pix = (((pix >> 12) & 0x0F) << 8)
			     | (((pix >> 7) & 0x0F) << 4)
			     | ((pix >> 1) & 0x0F);
			if(pix == 0x0FFF)
				pix = pix | 0xF000;	/* White s( = 15) */
			else
				pix = pix | 0x8000;	/* White ȊO( = 8) */
			*gra_vram++ = pix;
		}
	}

	while(1);
}
